@@ -27,9 +27,10 @@
  * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  */
+#include <cstring>
 #include <sys/time.h>
 #include <iostream>
-#include <map>
+#include <ext/hash_map>
 #include <list>
 #include <vector>
 #include <fstream>
@@ -42,11 +43,22 @@
 #include "nmv-date-utils.h"
 #include "nmv-safe-ptr-utils.h"
 
+using __gnu_cxx::hash_map ;
+using __gnu_cxx::hash ;
+
 namespace nemiver {
 namespace common {
 
 using namespace std ;
 
+struct Eqstr
+{
+    bool operator() (const char* s1, const char* s2) const
+    {
+        return strcmp (s1, s2) == 0;
+    }
+};
+
 static enum LogStream::StreamType s_stream_type = LogStream::COUT_STREAM ;
 static enum LogStream::LogLevel s_level_filter = LogStream::LOG_LEVEL_NORMAL ;
 static bool s_is_active = true ;
@@ -201,7 +213,7 @@ struct LogStream::Priv
     //the stack of default domains name
     //to consider when logging functions don't
     //specify the domain name in their parameters
-    list<UString> default_domains ;
+    list<string> default_domains ;
 
     //the list of domains (keywords) this stream
     //is allowed to log against. (It is a map, just for speed purposes)
@@ -209,14 +221,14 @@ struct LogStream::Priv
     //Logging domains are just keywords associated to the messages that are
     //going to be logged. This helps in for filtering the messages that
     //are to be logged or not.
-    std::map<UString, bool> allowed_domains ;
+    hash_map<const char*, bool, hash<const char*>, Eqstr > allowed_domains ;
 
     //the log level of this log stream
     enum LogStream::LogLevel level ;
 
     std::vector<UString> enabled_domains_from_env ;
 
-    Priv (const UString &a_domain=NMV_GENERAL_DOMAIN) :
+    Priv (const string &a_domain=NMV_GENERAL_DOMAIN) :
             stream_type (LogStream::COUT_STREAM),
             level (LogStream::LOG_LEVEL_NORMAL)
     {
@@ -245,14 +257,14 @@ struct LogStream::Priv
         return s_domain_filter ;
     }
 
-    bool is_logging_allowed (const UString &a_domain)
+    bool is_logging_allowed (const std::string &a_domain)
     {
         if (!LogStream::is_active ())
             return false ;
 
         //check domain
         if (allowed_domains.find ("all") == allowed_domains.end ()) {
-            if (allowed_domains.find (a_domain) == allowed_domains.end ()) {
+            if (allowed_domains.find (a_domain.c_str ()) == allowed_domains.end ()) {
                 return false ;
             }
         }
@@ -272,6 +284,9 @@ struct LogStream::Priv
     void load_enabled_domains_from_env ()
     {
         char *str = getenv ("nmv_log_domains") ;
+        if (!str) {
+            str = getenv ("NMV_LOG_DOMAINS") ;
+        }
         if (!str) {return;}
         UString domains_str = Glib::locale_to_utf8 (str) ;
         enabled_domains_from_env = domains_str.split (" ") ;
@@ -337,7 +352,7 @@ LogStream::default_log_stream ()
 }
 
 LogStream::LogStream (enum LogLevel a_level,
-                      const UString &a_domain) :
+                      const string &a_domain) :
     m_priv (new LogStream::Priv (a_domain))
 {
 
@@ -373,20 +388,20 @@ LogStream::~LogStream ()
 }
 
 void
-LogStream::enable_domain (const UString &a_domain,
+LogStream::enable_domain (const string &a_domain,
                           bool a_do_enable)
 {
     if (a_do_enable) {
-        m_priv->allowed_domains[a_domain] = true ;
+        m_priv->allowed_domains[a_domain.c_str ()] = true ;
     } else {
-        m_priv->allowed_domains.erase (a_domain) ;
+        m_priv->allowed_domains.erase (a_domain.c_str ()) ;
     }
 }
 
 bool
-LogStream::is_domain_enabled (const UString &a_domain)
+LogStream::is_domain_enabled (const std::string &a_domain)
 {
-    if (m_priv->allowed_domains.find (a_domain)
+    if (m_priv->allowed_domains.find (a_domain.c_str ())
         != m_priv->allowed_domains.end ()) {
         return true ;
     }
@@ -394,7 +409,7 @@ LogStream::is_domain_enabled (const UString &a_domain)
 }
 
 LogStream&
-LogStream::write (const char* a_buf, long a_buflen, const UString &a_domain)
+LogStream::write (const char* a_buf, long a_buflen, const string &a_domain)
 {
     if (!m_priv->is_logging_allowed (a_domain))
         return *this ;
@@ -417,7 +432,7 @@ LogStream::write (const char* a_buf, long a_buflen, const UString &a_domain)
 }
 
 LogStream&
-LogStream::write (int a_msg, const UString &a_domain)
+LogStream::write (int a_msg, const string &a_domain)
 {
     if (!m_priv || !m_priv->sink)
         return *this ;
@@ -435,7 +450,7 @@ LogStream::write (int a_msg, const UString &a_domain)
 
 LogStream&
 LogStream::write (double a_msg,
-                  const UString &a_domain)
+                  const string &a_domain)
 {
     if (!m_priv || !m_priv->sink)
         return *this ;
@@ -453,7 +468,7 @@ LogStream::write (double a_msg,
 
 LogStream&
 LogStream::write (char a_msg,
-                  const UString &a_domain)
+                  const string &a_domain)
 {
     if (!m_priv || !m_priv->sink)
         return *this ;
@@ -470,7 +485,7 @@ LogStream::write (char a_msg,
 }
 
 void
-LogStream::push_domain (const UString &a_domain)
+LogStream::push_domain (const string &a_domain)
 {
     m_priv->default_domains.push_front (a_domain) ;
 }
@@ -485,7 +500,7 @@ LogStream::pop_domain ()
 }
 
 LogStream&
-LogStream::write (const UString &a_msg, const UString &a_domain)
+LogStream::write (const UString &a_msg, const string &a_domain)
 {
     return write (a_msg.c_str (), a_msg.bytes (), a_domain) ;
 }
