@@ -15,7 +15,7 @@
  *See the GNU General Public License for more details.
  *
  *You should have received a copy of the
- *GNU General Public License along with Goupil;
+ *GNU General Public License along with Nemiver;
  *see the file COPYING.
  *If not, write to the Free Software Foundation,
  *Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
@@ -24,9 +24,9 @@
  */
 #include <gconf/gconf-client.h>
 #include "nmv-i-conf-mgr.h"
-#include "nmv-ustring.h"
-#include "nmv-exception.h"
-#include "nmv-safe-ptr-utils.h"
+#include "common/nmv-ustring.h"
+#include "common/nmv-exception.h"
+#include "common/nmv-safe-ptr-utils.h"
 
 NEMIVER_BEGIN_NAMESPACE (nemiver)
 
@@ -41,17 +41,14 @@ class GConfMgr : public IConfMgr {
 
 public:
 
-    void get_info (Info &a_info) const ;
-
-    void do_init () ;
-
-    GConfMgr () ;
+    GConfMgr (DynamicModule *a_dynmod) ;
     virtual ~GConfMgr () ;
 
     void set_key_dir_to_notify (const UString &a_key_dir) ;
+    void add_key_to_notify (const UString &a_key);
 
     bool get_key_value (const UString &a_key, UString &a_value) ;
-    void set_key_value (const UString &a_key, const UString &a_key) ;
+    void set_key_value (const UString &a_key, const UString &a_value) ;
 
     bool get_key_value (const UString &a_key, bool &a_value) ;
     void set_key_value (const UString &a_key, bool a_value) ;
@@ -78,27 +75,14 @@ struct GErrorUnref {
 
 typedef SafePtr<GError, GErrorRef, GErrorUnref> GErrorSafePtr ;
 
-
-void
-GConfMgr::get_info (Info &a_info) const
-{
-    a_info.module_name = "GConfMgr" ;
-    a_info.module_description =
-        "A GConf implementation of the IConfMgr interface" ;
-    a_info.module_version = "1.0" ;
-}
-
-void
-GConfMgr::do_init ()
-{
-}
-
 void
 client_notify_func (GConfClient *a_client,
                     const char* a_key,
                     GConfValue *a_value,
                     GConfMgr *a_conf_mgr)
 {
+    NEMIVER_TRY
+
     THROW_IF_FAIL (a_client) ;
     THROW_IF_FAIL (a_key) ;
     THROW_IF_FAIL (a_value) ;
@@ -132,13 +116,32 @@ client_notify_func (GConfClient *a_client,
             return ;
     }
     a_conf_mgr->value_changed_signal ().emit (a_key, value) ;
+
+    NEMIVER_CATCH_NOX
+}
+
+void
+client_notify_add_func (GConfClient *a_client,
+                        guint a_cnxn_id,
+                        GConfEntry *a_entry,
+                        GConfMgr *a_conf_mgr)
+{
+    THROW_IF_FAIL (a_client) ;
+    THROW_IF_FAIL (a_entry) ;
+    THROW_IF_FAIL (a_conf_mgr) ;
+    if (a_cnxn_id) {}
+
+    client_notify_func (a_client,
+                        a_entry->key,
+                        a_entry->value,
+                        a_conf_mgr);
 }
 
 void
 GConfMgr::set_key_dir_to_notify (const UString &a_key_dir)
 {
     THROW_IF_FAIL (m_gconf_client) ;
-    GError *err=NULL ;
+    GError *err=0;
     gconf_client_add_dir (m_gconf_client,
                           a_key_dir.c_str (),
                           GCONF_CLIENT_PRELOAD_NONE,
@@ -148,8 +151,25 @@ GConfMgr::set_key_dir_to_notify (const UString &a_key_dir)
     LOG_DD ("watching key for notification: '" << a_key_dir << "'") ;
 }
 
-GConfMgr::GConfMgr () :
-    m_gconf_client (NULL)
+void
+GConfMgr::add_key_to_notify (const UString &a_key)
+{
+    THROW_IF_FAIL (m_gconf_client) ;
+    GError *err=0;
+    gconf_client_notify_add (m_gconf_client,
+                             a_key.c_str (),
+                             (GConfClientNotifyFunc) client_notify_add_func,
+                             this,
+                             NULL,
+                             &err) ;
+    GErrorSafePtr error (err) ;
+    THROW_IF_FAIL2 (!error, error->message) ;
+    LOG_DD ("watching key for notification: '" << a_key << "'") ;
+}
+
+GConfMgr::GConfMgr (DynamicModule *a_dynmod) :
+    IConfMgr (a_dynmod),
+    m_gconf_client (0)
 {
     m_gconf_client = gconf_client_get_default () ;
     THROW_IF_FAIL (m_gconf_client) ;
@@ -303,13 +323,42 @@ GConfMgr::value_changed_signal ()
     return m_value_changed_signal ;
 }
 
+using nemiver::common::DynModIfaceSafePtr ;
+class GConfMgrModule : public DynamicModule {
+
+public:
+    void get_info (Info &a_info) const
+    {
+        a_info.module_name = "GConfMgr" ;
+        a_info.module_description =
+            "A GConf implementation of the IConfMgr interface" ;
+        a_info.module_version = "1.0" ;
+    }
+
+    /// \brief module init routinr
+    void do_init ()
+    {
+    }
+
+    bool lookup_interface (const std::string &a_iface_name,
+                           DynModIfaceSafePtr &a_iface)
+    {
+        if (a_iface_name == "IConfMgr") {
+            a_iface.reset (new GConfMgr (this)) ;
+        } else {
+            return false ;
+        }
+        return true ;
+    }
+};//end class GConfMgrModule
+
 NEMIVER_END_NAMESPACE (nemiver)
 
 extern "C" {
 bool
 NEMIVER_API nemiver_common_create_dynamic_module_instance (void **a_new_instance)
 {
-    *a_new_instance = new nemiver::GConfMgr () ;
+    *a_new_instance = new nemiver::GConfMgrModule () ;
     return (*a_new_instance != 0) ;
 }
 
