@@ -59,7 +59,9 @@ struct CallStackCols : public Gtk::TreeModelColumnRecord {
     Gtk::TreeModelColumn<Glib::ustring> location ;
     Gtk::TreeModelColumn<Glib::ustring> function_name ;
     Gtk::TreeModelColumn<Glib::ustring> function_args;
+    Gtk::TreeModelColumn<Glib::ustring> frame_index_caption;
     Gtk::TreeModelColumn<int> frame_index;
+    Gtk::TreeModelColumn<bool> is_expansion_row;
 
     enum Index {
         LOCATION=0,
@@ -72,7 +74,9 @@ struct CallStackCols : public Gtk::TreeModelColumnRecord {
         add (location) ;
         add (function_name) ;
         add (function_args) ;
+        add (frame_index_caption) ;
         add (frame_index) ;
+        add (is_expansion_row) ;
     }
 };//end cols
 
@@ -83,34 +87,63 @@ columns ()
     return s_cols ;
 }
 
+static const UString CONF_KEY_NEMIVER_CALLSTACK_EXPANSION_CHUNK =
+    "/apps/nemiver/dbgperspective/callstack-expansion-chunk";
+
 struct CallStack::Priv {
     IDebuggerSafePtr debugger ;
     IWorkbench& workbench;
     IPerspective& perspective;
     vector<IDebugger::Frame> frames ;
-    map<int, list<IDebugger::VariableSafePtr> > m_params;
+    map<int, list<IDebugger::VariableSafePtr> > params;
     Glib::RefPtr<Gtk::ListStore> store ;
     SafePtr<Gtk::TreeView> widget ;
     bool waiting_for_stack_args ;
     bool in_set_cur_frame_trans ;
     IDebugger::Frame cur_frame ;
-    int cur_frame_index ;
+    int cur_frame_index;
+    unsigned nb_frames_expansion_chunk;
+    unsigned max_frames_to_show;
     sigc::signal<void, int, const IDebugger::Frame&> frame_selected_signal ;
     sigc::connection on_selection_changed_connection ;
     Gtk::Widget *callstack_menu ;
     Glib::RefPtr<Gtk::ActionGroup> call_stack_action_group;
 
-    Priv (IDebuggerSafePtr a_dbg, IWorkbench& a_workbench, IPerspective& a_perspective) :
+    Priv (IDebuggerSafePtr a_dbg,
+          IWorkbench& a_workbench,
+          IPerspective& a_perspective) :
         debugger (a_dbg),
         workbench (a_workbench),
         perspective (a_perspective),
         waiting_for_stack_args (false),
         in_set_cur_frame_trans (false),
         cur_frame_index (-1),
+        nb_frames_expansion_chunk (5),
+        max_frames_to_show (nb_frames_expansion_chunk),
         callstack_menu (0)
     {
         connect_debugger_signals () ;
         init_actions ();
+        init_conf ();
+    }
+
+    void init_conf ()
+    {
+        IConfMgrSafePtr conf_mgr = workbench.get_configuration_manager ();
+        if (!conf_mgr)
+            return;
+
+        int chunk = 0;
+        conf_mgr->get_key_value (CONF_KEY_NEMIVER_CALLSTACK_EXPANSION_CHUNK,
+                                 chunk);
+        if (chunk) {
+            nb_frames_expansion_chunk = chunk;
+            max_frames_to_show = chunk;
+        }
+        conf_mgr->add_key_to_notify
+                            (CONF_KEY_NEMIVER_CALLSTACK_EXPANSION_CHUNK);
+        conf_mgr->value_changed_signal ().connect
+            (sigc::mem_fun (*this, &Priv::on_config_value_changed_signal));
     }
 
     void init_actions ()
@@ -134,14 +167,17 @@ struct CallStack::Priv {
             Gtk::ActionGroup::create ("callstack-action-group") ;
         call_stack_action_group->set_sensitive (true) ;
         int num_actions =
-            sizeof (s_call_stack_action_entries)/sizeof (ui_utils::ActionEntry) ;
+            sizeof (s_call_stack_action_entries)
+                /
+            sizeof (ui_utils::ActionEntry) ;
 
         ui_utils::add_action_entries_to_action_group
             (s_call_stack_action_entries,
              num_actions,
              call_stack_action_group) ;
 
-        workbench.get_ui_manager ()->insert_action_group (call_stack_action_group);
+        workbench.get_ui_manager ()->insert_action_group
+                                                (call_stack_action_group);
     }
 
     Gtk::Widget* load_menu (UString a_filename, UString a_widget_name)
@@ -164,7 +200,7 @@ struct CallStack::Priv {
     {
         if (!callstack_menu) {
             callstack_menu = load_menu ("callstackpopup.xml",
-                    "/CallStackPopup");
+                                        "/CallStackPopup");
             THROW_IF_FAIL (callstack_menu);
         }
         return callstack_menu;
@@ -273,10 +309,21 @@ struct CallStack::Priv {
 
         Gtk::TreeModel::iterator row_iter =
                 store->get_iter (selected_rows.front ()) ;
+
+
         cur_frame_index = (*row_iter)[columns ().frame_index] ;
         cur_frame = frames[cur_frame_index] ;
         THROW_IF_FAIL (cur_frame.level () >= 0) ;
         in_set_cur_frame_trans = true ;
+
+        //if the selected row is the "expand number of stack lines" row, trigger
+        //a redraw of the with more raws.
+        if ((*row_iter)[columns ().is_expansion_row]) {
+            max_frames_to_show += nb_frames_expansion_chunk;
+            set_frame_list (frames, params);
+            return;
+        }
+
         LOG_DD ("frame selected: '"<<  (int) cur_frame_index << "'") ;
         LOG_DD ("frame level: '" << (int) cur_frame.level () << "'") ;
         debugger->select_frame (cur_frame_index) ;
@@ -284,6 +331,16 @@ struct CallStack::Priv {
         NEMIVER_CATCH
     }
 
+    void on_config_value_changed_signal (const UString &a_key,
+                                         IConfMgr::Value &a_val)
+    {
+        LOG_DD ("key " << a_key << " changed");
+        if (a_key == CONF_KEY_NEMIVER_CALLSTACK_EXPANSION_CHUNK) {
+            nb_frames_expansion_chunk = boost::get<int> (a_val);
+            max_frames_to_show = nb_frames_expansion_chunk;
+        }
+    }
+
     void connect_debugger_signals ()
     {
         THROW_IF_FAIL (debugger) ;
@@ -301,6 +358,7 @@ struct CallStack::Priv {
     void on_call_stack_button_press_signal (GdkEventButton *a_event)
     {
         LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
         NEMIVER_TRY
 
         // right-clicking should pop up a context menu
@@ -315,15 +373,20 @@ struct CallStack::Priv {
     {
         THROW_IF_FAIL (a_event);
         THROW_IF_FAIL (widget);
+
         Gtk::Menu *menu = dynamic_cast<Gtk::Menu*> (get_call_stack_menu ()) ;
         THROW_IF_FAIL (menu) ;
-        // only pop up a menu if a row exists at that position
+
+        //only pop up a menu if a row exists at that position
         Gtk::TreeModel::Path path;
-        Gtk::TreeViewColumn* p_column = NULL;
+        Gtk::TreeViewColumn* column=NULL;
         int cell_x=0, cell_y=0;
-        if (widget->get_path_at_pos(static_cast<int>(a_event->x),
-                    static_cast<int>(a_event->y), path, p_column, cell_x,
-                    cell_y)) {
+        if (widget->get_path_at_pos (static_cast<int> (a_event->x),
+                                     static_cast<int> (a_event->y),
+                                     path,
+                                     column,
+                                     cell_x,
+                                     cell_y)) {
             menu->popup (a_event->button, a_event->time) ;
         }
     }
@@ -338,13 +401,14 @@ struct CallStack::Priv {
         map<int, list<IDebugger::VariableSafePtr> >::const_iterator params_iter;
         // convert list of stack frames to a string (FIXME: maybe Frame should
         // just implement operator<< ?
-        for (frame_iter=frames.begin(), params_iter=m_params.begin();
-                frame_iter != frames.end(); ++frame_iter)
+        for (frame_iter= frames.begin(), params_iter = params.begin();
+             frame_iter != frames.end ();
+             ++frame_iter)
         {
             frame_stream << "#" << UString::from_int(i++) << "  " <<
                 frame_iter->function_name () << " (";
             // if the params map exists, add the function params to the stack trace
-            if (params_iter != m_params.end())
+            if (params_iter != params.end())
             {
                 for (list<IDebugger::VariableSafePtr>::const_iterator it =
                         params_iter->second.begin();
@@ -380,8 +444,8 @@ struct CallStack::Priv {
         Gtk::TreeView *tree_view = new Gtk::TreeView (store) ;
         THROW_IF_FAIL (tree_view) ;
         widget.reset (tree_view) ;
-        tree_view->append_column (_("Frame"), columns ().frame_index) ;
-        tree_view->append_column (_("Line"), columns ().location) ;
+        tree_view->append_column (_("Frame"), columns ().frame_index_caption) ;
+        tree_view->append_column (_("Location"), columns ().location) ;
         tree_view->append_column (_("Function"), columns ().function_name) ;
         tree_view->append_column (_("Arguments"), columns ().function_args) ;
         tree_view->set_headers_visible (true) ;
@@ -424,16 +488,26 @@ struct CallStack::Priv {
 
         // save the list of params around so that we can use it when converting
         // to a string later
-        m_params = a_params;
+        params = a_params;
 
         Gtk::TreeModel::iterator store_iter ;
-        for (unsigned int i = 0 ; i < a_frames.size () ; ++i) {
+        unsigned nb_frames = MIN (a_frames.size (), max_frames_to_show);
+        unsigned i = 0;
+        for (i = 0; i < nb_frames; ++i) {
             store_iter = store->append () ;
+            (*store_iter)[columns ().is_expansion_row] = false;
             (*store_iter)[columns ().function_name] = a_frames[i].function_name ();
-            (*store_iter)[columns ().location] =
-                            a_frames[i].file_name () + ":"
-                            + UString::from_int (a_frames[i].line ()) ;
+            if (!a_frames[i].file_name ().empty ()) {
+                (*store_iter)[columns ().location] =
+                    a_frames[i].file_name () + ":"
+                    + UString::from_int (a_frames[i].line ()) ;
+            } else {
+                (*store_iter)[columns ().location] =
+                    a_frames[i].address ();
+            }
+
             (*store_iter)[columns ().frame_index] = i ;
+            (*store_iter)[columns ().frame_index_caption] =  UString::from_int (i) ;
             UString params_string = "(";
             map<int, list<IDebugger::VariableSafePtr> >::const_iterator iter ;
             list<IDebugger::VariableSafePtr>::const_iterator params_iter ;
@@ -459,6 +533,18 @@ struct CallStack::Priv {
             params_string += ")" ;
             (*store_iter)[columns ().function_args] = params_string ;
         }
+        if (a_frames.size () && i < a_frames.size () - 1) {
+            store_iter = store->append () ;
+            UString msg;
+            msg.printf (_("(Click here to see the next %d rows of the %d "
+                          "call stack rows)"),
+                        nb_frames_expansion_chunk,
+                        frames.size ());
+            (*store_iter)[columns ().frame_index_caption] = "...";
+            (*store_iter)[columns ().location] = msg ;
+            (*store_iter)[columns ().is_expansion_row] = true;
+        }
+
         Gtk::TreeView *tree_view =
             dynamic_cast<Gtk::TreeView*> (widget.get ()) ;
         THROW_IF_FAIL (tree_view) ;
@@ -479,8 +565,9 @@ struct CallStack::Priv {
     }
 };//end struct CallStack::Priv
 
-CallStack::CallStack (IDebuggerSafePtr &a_debugger, IWorkbench& a_workbench,
-        IPerspective& a_perspective)
+CallStack::CallStack (IDebuggerSafePtr &a_debugger,
+                      IWorkbench& a_workbench,
+                      IPerspective &a_perspective)
 {
     THROW_IF_FAIL (a_debugger) ;
     m_priv.reset (new Priv (a_debugger, a_workbench, a_perspective)) ;
