@@ -15,7 +15,7 @@
  *See the GNU General Public License for more details.
  *
  *You should have received a copy of the
- *GNU General Public License along with Goupil;
+ *GNU General Public License along with Nemiver;
  *see the file COPYING.
  *If not, write to the Free Software Foundation,
  *Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
@@ -26,11 +26,11 @@
 #include <vector>
 #include <libglademm.h>
 #include <gtkmm/dialog.h>
-#include "nmv-exception.h"
-#include "nmv-dialog.h"
-#include "nmv-env.h"
-#include "nmv-ustring.h"
+#include "common/nmv-exception.h"
+#include "common/nmv-env.h"
+#include "common/nmv-ustring.h"
 #include "nmv-ui-utils.h"
+#include "nmv-dialog.h"
 
 using namespace std ;
 using namespace nemiver::common ;
@@ -102,5 +102,13 @@ Dialog::run ()
     return m_priv->dialog->run () ;
 }
 
+void
+Dialog::hide ()
+{
+    THROW_IF_FAIL (m_priv) ;
+    THROW_IF_FAIL (m_priv->dialog) ;
+    return m_priv->dialog->hide () ;
+}
+
 }//end namespace nemiver
 
